#Copyright:   ?? 2006, Remi Goux

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

=begin 
    
    Name:       suapi.rb
    Version:    0.1b

    Install:    Drop into your Plugins directory. Relaunch SU.
             Switch on and off in SU Preferences/Extensions
    
    Description:   Add The complete ruby API to the Help menu
    
    Usage:      Help menu, then browse the class hierarchy
             
             In each class or module submenu:
             
                The first item displays the class/module name and genealogy.
                When selected, dump a summary for the classe to the ruby console
                
                The second item displays the class included modules, if any
                
                Further disabled items may list the class descendants, if any
                
                The remaining items display the class/module methods
                Class methods come first and use the 'Class::method' notation
                Instance methods follow
                When selected, dump a 1-line summary for the method to the ruby console
                
                The '#arity: n' comment shows the number of parameters
                Negative numbers optional or variable length parameters
                See Method#arity documentation for more

                The last item will open the related documentation in a web browser
                
    
    Features:   suapi.rb uses introspection to build its menus. This means that custom modules, classes and
             methods will show up, as long as they load before suapi. Likewise, any future API changes
             should be reflected.
             
             Undocumented modules, classes and methods are listed.
    
    Issues:      Under XP, there seems to be some kind of arbitrary limit to the number of menu items
            in a single menu and its submenus, so we bail out before we reach it (see SUAPIHelpMenu#add_items)
            After the limit, all items are grayed out. They do nothing but they still hilite :-]
            
            Under OS X, a single ampersand does not display in a menu (Array#& method)
            
   History:   Apr 27, 2006   first release
=end

class Module
         
   def has_classes?() constants.length > 0; end
   def own_methods() methods - Kernel.methods; end
   
   def sketchup_menu menu

      menu.add_item(name) { 
         puts "module #{name}"
         puts "include #{included_modules * ', '}" unless included_modules.empty?
         own_methods.sort.each {|m| dump_method m}
         instance_methods(false).sort.each {|m| dump_instance_method m}
      }
      menu.add_separator
      
      own_methods.sort.each do |m|
         menu.add_item("    #{name}::#{m}") { dump_method m }
      end

      instance_methods(false).sort.each do |m|
         menu.add_item("    #{m}") { dump_instance_method m }
      end

   end   
   
   def dump_method m
      puts '%-50s #arity:% 3d' % ["#{name}::#{m}", method(m).arity]
   end
   
   def dump_instance_method m
      puts '%-50s #arity:% 3d' % ["#{name}##{m}", instance_method(m).arity]
   end
   
end

class Class
         
   def superclasses() ancestors - included_modules; end
   
   def own_methods
      return methods if superclass.nil?
      methods - superclass.methods
   end
   
   def short_name() name.gsub(/^.+\:\:/, ''); end
   
   def sketchup_menu menu, descendants
      
      menu.add_item(superclasses[0..-2].join(' < ')) { 
         puts "class #{superclasses * ' < '}"
         puts "include #{included_modules * ', '}"
         own_methods.sort.each {|m| dump_method m }
         instance_methods(false).sort.each {|m| dump_instance_method m }
      }
      
      unless (result = included_modules[0..-2]).empty?
         menu.add_item_grayed "include #{result * ', '}"
      else
         menu.add_separator
      end
      
      descendants.each do |d|
         menu.add_item_grayed '   > ' + d.short_name
      end
      
      own_methods.sort.each do |m|
         menu.add_item("    #{short_name}::#{m}") { dump_method m }
      end
      
      instance_methods(false).sort.each do |m|
         menu.add_item("    #{m}") { dump_instance_method m }
      end
   end   
end

module Sketchup
   def Sketchup.is_mac?
      !(PLATFORM =~ /darwin/).nil?
   end
   def Sketchup.is_win?
      !(PLATFORM =~ /win32/).nil?
   end

   class Menu
      def add_item_grayed text
         item = add_item(text) {}
         set_validation_proc(item) { MF_GRAYED }
      end
   end
end

class SUAPIHelpMenu
   
   def initialize
      @current_module = nil
      @hierarchy = Hash.new {|h,k| h[k] = Array.new}
      @known_modules = Array.new
   end
   
   def hierarchy klass
      return [] if @current_module.nil?
      return @hierarchy[klass] unless @hierarchy.empty?
      for k in @current_module.constants.sort do
         klass = eval(@current_module.name + '::' + k)
         @hierarchy[klass.superclass] << klass
      end
      @hierarchy[klass]
   end
   
   def collect_modules klass
      for mod in klass.included_modules[0..-2] do
         @known_modules.push mod
      end
   end
   
   #limit the number of examined modules to the bare minimum under Windows
   def start_with
      return [Sketchup, Geom, UI, SKSocket, Test, ] if Sketchup.is_mac?
      return [Sketchup, Geom, UI, ]
   end
   
   def add_items menu

      menu.add_separator
      menu.add_item_grayed 'Sketchup API:'
      for mod in start_with do
         unless mod.has_classes?
            module_submenu menu, mod
            next
         end
         @current_module = mod
         submenu = menu.add_submenu mod.name
         module_submenu submenu, mod
         submenu.add_separator
         mod.constants.sort.each do |klassname|
            klass_submenu submenu, eval("#{mod.name}::#{klassname}")
         end
         @hierarchy.clear
      end
      
      menu.add_separator
      
      #Under XP, there seems to be some kind of arbitrary limit to the number of menu items
      #in a single menu and its submenus, so we bail out before we reach it (for now)
      #After the limit, all items are grayed out. They do nothing but they still hilite :-]
      
      #swap the # below to use in full anyway
      #if true 
      if Sketchup.is_mac?
      
         #it is business as usual with Mac OS X, though, so we continue
         
         @current_module = nil
         
         menu.add_item_grayed 'Ruby Extensions:'
         for klass in [Array, Length, Numeric, Set, String, ] do
            klass_submenu menu, klass
         end
         
         menu.add_separator
         menu.add_item_grayed 'Included Modules:'
         for mod in @known_modules.uniq do
            module_submenu menu, mod
         end
      
      end
      
   end
   
   def klass_submenu menu, klass
      collect_modules klass
      menu = menu.add_submenu(shortClassName = klass.short_name)
      klass.sketchup_menu menu, hierarchy(klass)
      documentation_item menu, shortClassName
   end
   
   def module_submenu menu, mod
      menu = menu.add_submenu mod.name
      mod.sketchup_menu menu
      documentation_item menu, mod.name
   end
   
   def documentation_item menu, shortName
      file = Sketchup::find_support_file("Ruby-#{shortName}.html", 'Ruby/Docs/')
      url = if file.nil? then if @current_module.nil?
            "http://www.ruby-doc.org/core/classes/#{shortName}.html"
         else ''; end
      else "file://#{file}"; end
      label = 'Open Documentation in Browser'
      menu.add_separator
      if url.empty?
         menu.add_item_grayed label
      else
         menu.add_item(label) { UI::openURL url }
      end
   end
   
   def load
      Sketchup.send_action 'showRubyPanel:'
      add_items UI::menu('Help')
      self
   end
   
   private_class_method :allocate, :new
   @@instance = new.load unless @@instance ||= nil
   
end

require 'extensions'

class SUAPIHelpMenuExtension <  SketchupExtension
   def initialize
      super 'Ruby API Help Menu', 'suapi.rb'
      self.description = <<-'DESC'
Add The complete ruby API to the Help menu.
DESC
        self.version = '1.0b'
      self.creator = 'Remi Goux'
        self.copyright = '??2006, Remi Goux'
   end
end
Sketchup.register_extension SUAPIHelpMenuExtension.new, true


